/*
*     Program SINAL6.C:

*    20.9.99; Einfhrung neuer Parameter und Manipulationsmglichkeiten:
*
*    1. Form des intra-period-Envelope (Profil):
*       - Zeitpunkt des Intensittsgipfels der Periode
*         (in % der pitch period)
*
*    2. Intra-period formant movements
*       - Prozent der Zu-bzw. Abnahme des Formanten
*
*    3. Formant intensity in % der Gesamtintensitt
*       Jeweils bei den Formantwerten notieren
*       (statt Bandbreiten im ASL)
*
*    4. Die Zwischendatei mit den einzelnen Pitch-Perioden
*       wird explizit ausgegeben und kann nacheditiert werden.
*
*    ________________________________________________________________

*     8.3.99; Frames mit der Dauer 0 knnen gesetzt werden, um
*     die lineare Interpolation zu unterbrechen. Dadurch ist es z.B.
*     mglich, scharfe Oktavsprnge zu generieren.

*     9.3.99: Die stimmhaften Segmente werden jetzt grundstzlich
*     auf ganze pitch-periods abgerundet. Das letzte Segment wird
*     gleich behandelt wie alle brigen, die letzte pitch period
*     fllt nicht weg. Die lineare Interpolation kann nicht
*     mehr durch Aufrundung auf volle F0-periods das target
*     berschiessen.

*     Das Programm liest Ascii-Dumps aus ASL ein und synthetisiert
*     wav-Dateien. Es werden 5 Formanten ausgewertet.
*     Bei der Synthese werden die Formanten hergestellt und
*     mit der Grundfrequenz amplitudenmoduliert. Die Phasenlage der
*     Formanten erhlt mit jeder Pitch-Periode einen Reset.
*     Die maximale Intensitt jeder Periode wird durch den
*     Intensittswert im ASCII-Dump gesetzt.
*     Die Form der Grundfrequenzmodulation wird durch folgende Parameter
*     festgelegt:   -  % Offset (unmodulierter Amplitudenanteil)
*                   -  Anstieg = 1/4 Periode des ersten Formanten
*
*/


#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <math.h>

/******************************************************************/

/* globale Variablen */

char string[100], neutext[100], text[100];
FILE  *asc, *zwi, *fil;


int zeile (FILE *f)
                 /* Zeile lesen Unterprogramm */
          {
          int blanks;

           do {
               if (fgets(string,100,f)==0) exit(0) ;
               neutext[0]=(char)0;
               sscanf (string,"%[^\n]s",neutext);
               blanks=0;
               while((char)neutext[blanks++] == ' ');
               strcpy(text,neutext+blanks-1);
              } while (strlen(text) == 0) ;    /* Leerzeile */
           return(1);
          }


void main(int argc,char *argv[])
{
     unsigned long int nframes, nperiods;
     double  pkn, pkv,phn, p1, p2, len;
     double sampr1;
     double intv1, intv2, intvoi, intn1, intn2, intnoi1, intnoi2;
     double phan1, phan2, pha_noi1, pha_noi2;
     double dur1, dur2;
     double p1_1, p1_2, p2_1, p2_2;
     double f0_1,f1_1,f2_1,f3_1,f4_1,f5_1;
     double f0_2,f1_2,f2_2,f3_2,f4_2,f5_2;
     double b0_1,b1_1,b2_1,b3_1,b4_1,b5_1;
     double b0_2,b1_2,b2_2,b3_2,b4_2,b5_2;
     double pi2, period,F0,B0;
     double F1a,F2a,F3a,F4a,F5a,B1a,B2a,B3a,B4a,B5a;
     double F1e,F2e,F3e,F4e,F5e,B1e,B2e,B3e,B4e,B5e;
     double fra_start, fra_stop, delta, time;
     double prof1, prof2;
     double slo_pkv, slo_pkn, slo_phn, slo_p1, slo_p2;
     double slo_f0, slo_f1, slo_f2, slo_f3, slo_f4, slo_f5;
     double slo_b0, slo_b1, slo_b2, slo_b3, slo_b4, slo_b5;
     double b[6], f[6];
     FILE *asc, *zwi;

/******************* evaluate arguments ****************************/

   if (argc < 3)
      {
        printf("\nFor SINAMP.EXE 10 arguments are required:");
        printf("\n       Ascii input file");
        printf("\n       sampling rate of parameter file");
        printf("\n       Ascii pitch period file");
      }

   asc = fopen(argv[1],"rt");                  /* open formants file */
         if (asc == 0)
             { printf("\nno parameter file found\n");
               exit (-1);
             }

   sscanf (argv[2],"%lf",&sampr1)  ;        /* formants file Sampling rate */

   zwi = fopen(argv[3],"wt");               /* pitch period file */

   pi2 = (double) 2. * acos(-1.);

/*******************  start reading formants file  ***********************/

zeile(asc);               /* skip header */
zeile(asc);               /* 1st start frame */
sscanf (string,"%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf",
                &pkv,&pkn,&phn,&len,&p1,&p2,
                &f[0],&b[0],
                &f[1],&b[1],
                &f[2],&b[2],
                &f[3],&b[3],
                &f[4],&b[4],
                &f[5],&b[5]);
dur1      = (double)len / sampr1;       /* duration of first frame */
intv1      = (double)pkv;               /* intensity of first frame */
intn1      = (double)pkn;               /* intensity of first frame */
phan1     = (double)phn;
p1_1      = (double)p1;
p2_1      = (double)p2;
f0_1      = (double)f[0];               /* initial pitch */
f1_1      = (double)f[1];
f2_1      = (double)f[2];
f3_1      = (double)f[3];
f4_1      = (double)f[4];
f5_1      = (double)f[5];
b0_1      = (double)b[0];
b1_1      = (double)b[1];
b2_1      = (double)b[2];
b3_1      = (double)b[3];
b4_1      = (double)b[4];
b5_1      = (double)b[5];
nperiods  = 0;                          /*   number of synthesis periods */
nframes   = 1;                          /*   number of frames processed */
fra_start = 0;                          /*   start time of first frame  */
time      = 0;                          /*   start time of output data  */
/*************** printf ("\n  starting frames loop"); getch(); *******/
while   (zeile(asc)!=0)                        /* frames loop */
        {
         nframes++;
         if (dur1 == 0)
            {
                sscanf (string,
                "%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf",
                &pkv,&pkn,&phn,&len,&p1,&p2,
                &f[0],&b[0],
                &f[1],&b[1],
                &f[2],&b[2],
                &f[3],&b[3],
                &f[4],&b[4],
                &f[5],&b[5]);
      dur1      = (double)len / sampr1;       /* duration of first frame */
      intv1     = (double)pkv;               /* intensity of first frame */
      intn1     = (double)pkn;               /* intensity of first frame */
      phan1     = (double)phn;               /* phase noise */
      p1_1      = (double)p1;               /* profile of first frame */
      p2_1      = (double)p2;               /* profile of first frame */
      f0_1      = (double)f[0];               /* initial pitch */
      b0_1      = (double)b[0];               /* relative intensity of pitch */
      f1_1      = (double)f[1];
      f2_1      = (double)f[2];
      f3_1      = (double)f[3];
      f4_1      = (double)f[4];
      f5_1      = (double)f[5];
      b1_1      = (double)b[1];
      b2_1      = (double)b[2];
      b3_1      = (double)b[3];
      b4_1      = (double)b[4];
      b5_1      = (double)b[5];

                zeile(asc) ;                  /* current end frame */

            }                                   /* end if dur==0 */

             printf ("\n  dur1 = %G",dur1);

                sscanf (string,
                "%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf%lf",
                &pkv,&pkn,&phn,&len,&p1,&p2,
                &f[0],&b[0],
                &f[1],&b[1],
                &f[2],&b[2],
                &f[3],&b[3],
                &f[4],&b[4],
                &f[5],&b[5]);
         dur2      = (double)len / sampr1;  /* duration of following frame */
         intv2     = (double)pkv;           /* intensity  */
         intn2     = (double)pkn;           /* intensity */
         phan2     = (double)phn;           /* phase noise */
         p1_2      = (double)p1;               /* profile of last frame */
         p2_2      = (double)p2;               /* profile of last frame */
         f0_2      = (double)f[0];            /* end pitch frequency */
         f1_2      = (double)f[1];
         f2_2      = (double)f[2];
         f3_2      = (double)f[3];
         f4_2      = (double)f[4];
         f5_2      = (double)f[5];
         b0_2      = (double)b[0];           /* rel pitch amplitude at end */
         b1_2      = (double)b[1];
         b2_2      = (double)b[2];
         b3_2      = (double)b[3];
         b4_2      = (double)b[4];
         b5_2      = (double)b[5];

         fra_stop = time + dur1 ;      /* end time of frame */
         slo_pkv   = (intv2-intv1) / dur1;  /* for linear interpolation */
         slo_pkn   = (intn2-intn1) / dur1;  /* for linear interpolation */
         slo_phn   = (phan2-phan1) / dur1;  /* for linear interpolation */
         slo_p1   = (p1_2-p1_1) / dur1;     /* for linear interpolation */
         slo_p2   = (p2_2-p2_1) / dur1;     /* for linear interpolation */
         slo_f0   = (f0_2-f0_1) / dur1;     /* for linear interpolation */
         slo_f1   = (f1_2-f1_1) / dur1;     /* for linear interpolation */
         slo_f2   = (f2_2-f2_1) / dur1;     /* for linear interpolation */
         slo_f3   = (f3_2-f3_1) / dur1;     /* for linear interpolation */
         slo_f4   = (f4_2-f4_1) / dur1;     /* for linear interpolation */
         slo_f5   = (f5_2-f5_1) / dur1;     /* for linear interpolation */
         slo_b0   = (b0_2-b0_1) / dur1;     /* for linear interpolation */
         slo_b1   = (b1_2-b1_1) / dur1;     /* for linear interpolation */
         slo_b2   = (b2_2-b2_1) / dur1;     /* for linear interpolation */
         slo_b3   = (b3_2-b3_1) / dur1;     /* for linear interpolation */
         slo_b4   = (b4_2-b4_1) / dur1;     /* for linear interpolation */
         slo_b5   = (b5_2-b5_1) / dur1;     /* for linear interpolation */
         if (f0_1 == 0) slo_f0 = 0;
         if (f0_1 == 0) slo_p1 = 0;
         if (f0_1 == 0) slo_p2 = 0;

/********* printf ("\n enter periods loop  %G",fra_stop); ***********/
         while (time <= fra_stop)            /* start pitch periods loop */
               {
                delta    = time - fra_start;        /* Abscissa        */

                    intvoi  = intv1 + (slo_pkv * delta); /* local intensity */
                    prof1   = p1_1 + (slo_p1 * delta); /* local profile */
                    prof2   = p2_1 + (slo_p2 * delta); /* local profile */
                    F0      = f0_1 + (slo_f0 * delta);  /* local pitch   */
                    B0      = b0_1 + (slo_b0 * delta);  /* rel F0 ampl. */
                    period  = 1 / F0;

                F1a  = f1_1 + (slo_f1 * delta);  /* local F1      */
                F2a  = f2_1 + (slo_f2 * delta);  /* local F2      */
                F3a  = f3_1 + (slo_f3 * delta);  /* local F3      */
                F4a  = f4_1 + (slo_f4 * delta);  /* local F4      */
                F5a  = f5_1 + (slo_f5 * delta);  /* local F5      */
                B1a  = b1_1 + (slo_b1 * delta);  /* local rel. F1 */
                B2a  = b2_1 + (slo_b2 * delta);  /* local rel. F2 */
                B3a  = b3_1 + (slo_b3 * delta);  /* local rel. F3 */
                B4a  = b4_1 + (slo_b4 * delta);  /* local rel. F4 */
                B5a  = b5_1 + (slo_b5 * delta);  /* local rel. F5 */
                intnoi1 = intn1 + (slo_pkn * delta);
                pha_noi1 = phan1 + (slo_phn * delta);
                delta = delta + period;
                F1e  = f1_1 + (slo_f1 * delta);  /* local F1      */
                F2e  = f2_1 + (slo_f2 * delta);  /* local F2      */
                F3e  = f3_1 + (slo_f3 * delta);  /* local F3      */
                F4e  = f4_1 + (slo_f4 * delta);  /* local F4      */
                F5e  = f5_1 + (slo_f5 * delta);  /* local F5      */
                B1e  = b1_1 + (slo_b1 * delta);  /* local rel. F1 */
                B2e  = b2_1 + (slo_b2 * delta);  /* local rel. F2 */
                B3e  = b3_1 + (slo_b3 * delta);  /* local rel. F3 */
                B4e  = b4_1 + (slo_b4 * delta);  /* local rel. F4 */
                B5e  = b5_1 + (slo_b5 * delta);  /* local rel. F5 */
                intnoi2 = intn1 + (slo_pkn * delta);
                pha_noi2 = phan1 + (slo_phn * delta);
 
                if ((time+period) <= fra_stop+.0000001)
                {
                nperiods++;
                fprintf (zwi,
"%4.0ld %9.6lf %11.8lf %5.0lf %4.2lf %4.2lf %8.4lf %3.0lf \n",
nperiods,time, period,intvoi, prof1, prof2, F0,    B0);
                fprintf (zwi,
"%5.0lf %5.0lf %5.3lf %5.3lf\n",
intnoi1,intnoi2,pha_noi1,pha_noi2);
                fprintf (zwi,
"%8.0lf %2.0lf %4.0lf %2.0lf %4.0lf %2.0lf %4.0lf %2.0lf %4.0lf %2.0lf \n",
                F1a,B1a,F2a,B2a,F3a,B3a,F4a,B4a,F5a,B5a);
                fprintf (zwi,
"%8.0lf %2.0lf %4.0lf %2.0lf %4.0lf %2.0lf %4.0lf %2.0lf %4.0lf %2.0lf \n",
                F1e,B1e,F2e,B2e,F3e,B3e,F4e,B4e,F5e,B5e);

                time = time + period;
                }
                else break;

/*
printf ("\n start %G %G %G %G %G",fra_start, fra_stop, time, period, F0);
printf ("\ntime %ld %ld", nframes, nperiods);
getch();
*/
               }   ;                         /* end periods loop */


              dur1  =     dur2   ;
              intv1  =    intv2  ;
              intn1  =    intn2  ;
              phan1 =     phan2  ;
              p1_1  =     p1_2   ;
              p2_1  =     p2_2   ;
              f0_1  =     f0_2   ;
              f1_1  =     f1_2   ;
              f2_1  =     f2_2   ;
              f3_1  =     f3_2   ;
              f4_1  =     f4_2   ;
              f5_1  =     f5_2   ;
              b0_1  =     b0_2   ;
              b1_1  =     b1_2   ;
              b2_1  =     b2_2   ;
              b3_1  =     b3_2   ;
              b4_1  =     b4_2   ;
              b5_1  =     b5_2   ;

              fra_start = time;

            }                                /* end frames loop */


}






  ;           /*   Programmende     */
